import { chemicalSchema } from '@/models/chemicalSchema';
import { apiPath } from '@/redux/apiPath';
import { axiosInstanceNewAPI } from '@/services/api/utils/axiosInstance';
import { Chemical } from '@/types/models';
import { getErrorMessage } from '@/utils/getErrorMessage';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { z } from 'zod';
import { ThunkConfig } from '@/types/store';
import {
  CHEMICALS_FETCHING_ERROR_PREFIX,
  MULTI_CHEMICALS_FETCHING_ERROR_PREFIX,
} from './chemicals.constants';

export const getChemicals = createAsyncThunk<Chemical[] | undefined, string, ThunkConfig>(
  'project/getChemicals',
  async (searchQuery, { rejectWithValue }) => {
    try {
      const response = await axiosInstanceNewAPI.get<Chemical[]>(
        apiPath.getChemicalsStringWithQuery(searchQuery),
      );

      const isDataValid = validateDataUsingZodSchema(response.data, z.array(chemicalSchema));

      return isDataValid ? response.data : undefined;
    } catch (error) {
      const errorMessage = getErrorMessage({ error, prefix: CHEMICALS_FETCHING_ERROR_PREFIX });
      return rejectWithValue(errorMessage);
    }
  },
);

export const getMultiChemicals = createAsyncThunk<void, string[], ThunkConfig>(
  'project/getMultChemicals',
  // eslint-disable-next-line consistent-return
  async (searchQueries, { dispatch, rejectWithValue }) => {
    try {
      const asyncGetChemicalsFunctions = searchQueries.map(searchQuery =>
        dispatch(getChemicals(searchQuery)),
      );

      await Promise.all(asyncGetChemicalsFunctions);
    } catch (error) {
      const errorMessage = getErrorMessage({
        error,
        prefix: MULTI_CHEMICALS_FETCHING_ERROR_PREFIX,
      });

      return rejectWithValue(errorMessage);
    }
  },
);
