import { SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { rootSelector } from '@/redux/root/root.selectors';
import { MultiSearchData } from '@/types/fetchDataState';
import { BioEntity, Drug } from '@/types/models';
import { createSelector } from '@reduxjs/toolkit';
import { currentSelectedSearchElement } from '../drawer/drawer.selectors';
import { currentModelIdSelector } from '../models/models.selectors';

export const drugsSelector = createSelector(rootSelector, state => state.drugs);

export const drugsDataSelector = createSelector(drugsSelector, drugs => drugs.data);

export const drugsDataListSelector = createSelector(drugsDataSelector, drugsData =>
  drugsData.map(d => d.data || []).flat(),
);

export const drugsForSelectedSearchElementSelector = createSelector(
  drugsSelector,
  currentSelectedSearchElement,
  (drugsState, currentSearchElement): MultiSearchData<Drug[]> | undefined =>
    drugsState.data.find(({ searchQueryElement }) => searchQueryElement === currentSearchElement),
);

export const loadingDrugsStatusSelector = createSelector(
  drugsForSelectedSearchElementSelector,
  state => state?.loading,
);

export const numberOfDrugsSelector = createSelector(
  drugsForSelectedSearchElementSelector,
  state => {
    if (!state || !state?.data) {
      return SIZE_OF_EMPTY_ARRAY;
    }

    return state.data.length && state?.data.map(e => e.targets.length)?.reduce((a, b) => a + b);
  },
);

export const searchedDrugsBioEntitesOfCurrentMapSelector = createSelector(
  drugsSelector,
  currentSelectedSearchElement,
  currentModelIdSelector,
  (drugsState, currentSearchElement, currentModelId): BioEntity[] => {
    return (drugsState?.data || [])
      .filter(({ searchQueryElement }) =>
        currentSearchElement ? searchQueryElement === currentSearchElement : true,
      )
      .map(({ data }) => data || [])
      .flat()
      .map(({ targets }) => targets.map(({ targetElements }) => targetElements))
      .flat()
      .flat()
      .filter(bioEntity => bioEntity.model === currentModelId);
  },
);

export const allDrugsBioEntitesOfAllMapsSelector = createSelector(
  drugsSelector,
  (drugsState): BioEntity[] => {
    return (drugsState?.data || [])
      .map(({ data }) => data || [])
      .flat()
      .map(({ targets }) => targets.map(({ targetElements }) => targetElements))
      .flat()
      .flat();
  },
);
