import { createSlice } from '@reduxjs/toolkit';
import { MAP_INITIAL_STATE } from './map.constants';
import {
  closeMapAndSetMainMapActiveReducer,
  closeMapReducer,
  initMapBackgroundsReducer,
  initMapPositionReducers,
  initMapSizeAndModelIdReducer,
  initOpenedMapsReducer,
  openMapAndSetActiveReducer,
  setActiveMapReducer,
  setLastPositionZoomReducer,
  setMapBackgroundReducer,
  setMapDataReducer,
  setMapPositionReducer,
  varyPositionZoomReducer,
} from './map.reducers';

const mapSlice = createSlice({
  name: 'map',
  initialState: MAP_INITIAL_STATE,
  reducers: {
    setMapData: setMapDataReducer,
    setActiveMap: setActiveMapReducer,
    openMapAndSetActive: openMapAndSetActiveReducer,
    closeMap: closeMapReducer,
    closeMapAndSetMainMapActive: closeMapAndSetMainMapActiveReducer,
    setMapPosition: setMapPositionReducer,
    varyPositionZoom: varyPositionZoomReducer,
    setMapBackground: setMapBackgroundReducer,
    setLastPositionZoom: setLastPositionZoomReducer,
  },
  extraReducers: builder => {
    initMapPositionReducers(builder);
    initMapSizeAndModelIdReducer(builder);
    initMapBackgroundsReducer(builder);
    initOpenedMapsReducer(builder);
  },
});

export const {
  setMapData,
  setActiveMap,
  openMapAndSetActive,
  closeMap,
  closeMapAndSetMainMapActive,
  setMapPosition,
  setMapBackground,
  varyPositionZoom,
  setLastPositionZoom,
} = mapSlice.actions;

export default mapSlice.reducer;
